# Print an optspec for argparse to handle cmd's options that are independent of any subcommand.
function __fish_freebsd_rustdate_global_optspecs
	string join \n c/config= as-version= j/jobs-cpu= J/jobs-net= b/basedir= w/workdir= s/server= h/help V/version
end

function __fish_freebsd_rustdate_needs_command
	# Figure out if the current invocation already has a command.
	set -l cmd (commandline -opc)
	set -e cmd[1]
	argparse -s (__fish_freebsd_rustdate_global_optspecs) -- $cmd 2>/dev/null
	or return
	if set -q argv[1]
		# Also print the command, so this can be used to figure out what it is.
		echo $argv[1]
		return 1
	end
	return 0
end

function __fish_freebsd_rustdate_using_subcommand
	set -l cmd (__fish_freebsd_rustdate_needs_command)
	test -z "$cmd"
	and return 1
	contains -- $cmd[1] $argv
end

complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_needs_command" -s c -l config -d 'Config file' -r -F
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_needs_command" -l as-version -d 'Act as if we\'re running a given version' -r
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_needs_command" -s j -l jobs-cpu -d 'How many CPU-bound threads to run in parallel (default numcpu up to 6)' -r
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_needs_command" -s J -l jobs-net -d 'How many network request to do in parallel (default 4)' -r
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_needs_command" -s b -l basedir -d 'Operate on system mounted on a given basedir' -r -f -a "(__fish_complete_directories)"
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_needs_command" -s w -l workdir -d 'Store working files in workdir' -r -f -a "(__fish_complete_directories)"
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_needs_command" -s s -l server -d 'Server to fetch updates from' -r -f -a "(__fish_print_hostnames)"
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_needs_command" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_needs_command" -s V -l version -d 'Print version'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_needs_command" -f -a "fetch" -d 'Fetch updates to current version'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_needs_command" -f -a "cron" -d 'Fetch updates to current version via cron'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_needs_command" -f -a "upgrade" -d 'Fetch upgrades to a new version'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_needs_command" -f -a "install" -d 'Install downloaded updates or upgrades'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_needs_command" -f -a "show-install" -d 'Show information about a pending install'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_needs_command" -f -a "show-merges" -d 'Show merged files from a pending upgrade'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_needs_command" -f -a "resolve-merges" -d 'Resolve conflicted merges for a pending upgrade'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_needs_command" -f -a "clean" -d 'Clean up stuff (not all stuff included)'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_needs_command" -f -a "check-sys" -d 'Check current system state against upstream expectation'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_needs_command" -f -a "check-fetch" -d 'Quick check of whether there might be a newer patch available'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_needs_command" -f -a "extract" -d 'Extract a file or subtree exactly from upstream'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_needs_command" -f -a "complete" -d 'Generate command completions for a given shell'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_needs_command" -f -a "dump-metadata" -d 'Dump out metadata info for a version.  (DEV)'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_needs_command" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand fetch" -l as-cron -d 'Run as a backend via `$0 cron`'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand fetch" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand cron" -l immediately -d 'Run without delay.  This is a hidden option to make dev easier'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand cron" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand upgrade" -s r -l release -d 'Release to upgrade to (e.g., 13.2-RELEASE)' -r
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand upgrade" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand install" -s n -l dry-run -d 'Just say what we\'d do, don\'t actually do it.'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand install" -s a -l all -d 'Proceed through all steps of installing an Upgrade'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand install" -s s -l no-sync -d 'Don\'t fsync() files during the install'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand install" -s j -l parallel -d 'Install files in parallel'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand install" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand show-install" -s v -l verbose -d 'Show full details (e.g., file lists)' -r -f -a "{all\t'All types',add\t'Added files',rm\t'Removed files',update\t'Updated files',change\t'Paths with changed type',merge\t'Merged files'}"
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand show-install" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand show-merges" -s u -l upstream -d 'Show diffs relative to new upstream instead of your current file'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand show-merges" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand resolve-merges" -s e -l exit -d 'Don\'t attempt resolution, just describe our state and exit'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand resolve-merges" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand clean" -s p -l pending -d 'Clean up info about a pending update (i.e., erase knowledge of a previous `fetch` or `upgrade`)'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand clean" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand check-sys" -s i -l ignore -d 'Ignore differences of some type (can be specified multiple times)' -r -f -a "{uid\t'Owning user',gid\t'Owning group',mode\t'Mode',flags\t'Flags',hash\t'File hash',target\t'Link target',missing\t'Missing files',type\t'Mismatched file types'}"
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand check-sys" -s p -l paths -d 'Include only matching paths (regex)' -r
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand check-sys" -s x -l exclude -d 'Exclude paths (regex)' -r
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand check-sys" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand check-fetch" -s q -l quiet -d 'Run quietly'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand check-fetch" -s c -l cron -d 'Add a random delay to run via cron'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand check-fetch" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand extract" -s n -l dry-run -d 'Just say what we\'d do, don\'t actually do it'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand extract" -s x -l regex -d 'Treat given paths as regular expressions instead of literals'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand extract" -s c -l only-components -d 'Attempt to filter down to installed components'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand extract" -s f -l force -d 'Forcibly overwrite existing files, even if they look the same'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand extract" -s s -l no-sync -d 'Don\'t fsync() files during the install'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand extract" -s j -l parallel -d 'Install files in parallel'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand extract" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand complete" -s s -l shell -d 'Shell to generate completions for' -r -f -a "{bash\t'',elvish\t'',fish\t'',powershell\t'',zsh\t''}"
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand complete" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand dump-metadata" -s v -l version -d 'Version to dump metadata for (e.g., 13.2-RELEASE)' -r
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand dump-metadata" -s d -l dir -d 'Directory to save the files into (must exist)' -r -f -a "(__fish_complete_directories)"
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand dump-metadata" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand help; and not __fish_seen_subcommand_from fetch cron upgrade install show-install show-merges resolve-merges clean check-sys check-fetch extract complete dump-metadata help" -f -a "fetch" -d 'Fetch updates to current version'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand help; and not __fish_seen_subcommand_from fetch cron upgrade install show-install show-merges resolve-merges clean check-sys check-fetch extract complete dump-metadata help" -f -a "cron" -d 'Fetch updates to current version via cron'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand help; and not __fish_seen_subcommand_from fetch cron upgrade install show-install show-merges resolve-merges clean check-sys check-fetch extract complete dump-metadata help" -f -a "upgrade" -d 'Fetch upgrades to a new version'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand help; and not __fish_seen_subcommand_from fetch cron upgrade install show-install show-merges resolve-merges clean check-sys check-fetch extract complete dump-metadata help" -f -a "install" -d 'Install downloaded updates or upgrades'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand help; and not __fish_seen_subcommand_from fetch cron upgrade install show-install show-merges resolve-merges clean check-sys check-fetch extract complete dump-metadata help" -f -a "show-install" -d 'Show information about a pending install'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand help; and not __fish_seen_subcommand_from fetch cron upgrade install show-install show-merges resolve-merges clean check-sys check-fetch extract complete dump-metadata help" -f -a "show-merges" -d 'Show merged files from a pending upgrade'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand help; and not __fish_seen_subcommand_from fetch cron upgrade install show-install show-merges resolve-merges clean check-sys check-fetch extract complete dump-metadata help" -f -a "resolve-merges" -d 'Resolve conflicted merges for a pending upgrade'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand help; and not __fish_seen_subcommand_from fetch cron upgrade install show-install show-merges resolve-merges clean check-sys check-fetch extract complete dump-metadata help" -f -a "clean" -d 'Clean up stuff (not all stuff included)'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand help; and not __fish_seen_subcommand_from fetch cron upgrade install show-install show-merges resolve-merges clean check-sys check-fetch extract complete dump-metadata help" -f -a "check-sys" -d 'Check current system state against upstream expectation'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand help; and not __fish_seen_subcommand_from fetch cron upgrade install show-install show-merges resolve-merges clean check-sys check-fetch extract complete dump-metadata help" -f -a "check-fetch" -d 'Quick check of whether there might be a newer patch available'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand help; and not __fish_seen_subcommand_from fetch cron upgrade install show-install show-merges resolve-merges clean check-sys check-fetch extract complete dump-metadata help" -f -a "extract" -d 'Extract a file or subtree exactly from upstream'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand help; and not __fish_seen_subcommand_from fetch cron upgrade install show-install show-merges resolve-merges clean check-sys check-fetch extract complete dump-metadata help" -f -a "complete" -d 'Generate command completions for a given shell'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand help; and not __fish_seen_subcommand_from fetch cron upgrade install show-install show-merges resolve-merges clean check-sys check-fetch extract complete dump-metadata help" -f -a "dump-metadata" -d 'Dump out metadata info for a version.  (DEV)'
complete -c freebsd-rustdate -n "__fish_freebsd_rustdate_using_subcommand help; and not __fish_seen_subcommand_from fetch cron upgrade install show-install show-merges resolve-merges clean check-sys check-fetch extract complete dump-metadata help" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
